package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestPORT {

  private static final int BYTE_VALUE = 0x10;
  private final String MNEMONIC = "PORT";

  @Test
  void testMnemonic() throws SyntaxError {
    PORT mno = new PORT(MNEMONIC + " 0x00");
    mno.checkArgument();
    assertEquals(BYTE_VALUE, mno.getByte());

    mno = new PORT(MNEMONIC + " 0x01");
    mno.checkArgument();
    assertEquals(BYTE_VALUE + 1, mno.getByte());

    mno = new PORT(MNEMONIC + " 0x0f");
    mno.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      PORT mno1 = new PORT(MNEMONIC);
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      PORT mno1 = new PORT(MNEMONIC + " 0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    assertEquals(PORT.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

  }
}
